# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'edit_event.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(413, 488)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label = QtWidgets.QLabel(Form)
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pb_set_to_current_time = QtWidgets.QPushButton(Form)
        self.pb_set_to_current_time.setObjectName("pb_set_to_current_time")
        self.horizontalLayout_2.addWidget(self.pb_set_to_current_time)
        self.cb_set_time_na = QtWidgets.QCheckBox(Form)
        self.cb_set_time_na.setObjectName("cb_set_time_na")
        self.horizontalLayout_2.addWidget(self.cb_set_time_na)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.lb_image_idx = QtWidgets.QLabel(Form)
        self.lb_image_idx.setObjectName("lb_image_idx")
        self.horizontalLayout_7.addWidget(self.lb_image_idx)
        self.sb_image_idx = QtWidgets.QSpinBox(Form)
        self.sb_image_idx.setMaximum(10000000)
        self.sb_image_idx.setObjectName("sb_image_idx")
        self.horizontalLayout_7.addWidget(self.sb_image_idx)
        self.pb_set_to_current_image_index = QtWidgets.QPushButton(Form)
        self.pb_set_to_current_image_index.setObjectName("pb_set_to_current_image_index")
        self.horizontalLayout_7.addWidget(self.pb_set_to_current_image_index)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.lbSubject = QtWidgets.QLabel(Form)
        self.lbSubject.setObjectName("lbSubject")
        self.horizontalLayout_4.addWidget(self.lbSubject)
        self.cobSubject = QtWidgets.QComboBox(Form)
        self.cobSubject.setObjectName("cobSubject")
        self.horizontalLayout_4.addWidget(self.cobSubject)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_5.addWidget(self.label_2)
        self.cobCode = QtWidgets.QComboBox(Form)
        self.cobCode.setObjectName("cobCode")
        self.horizontalLayout_5.addWidget(self.cobCode)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.lb_frame_idx = QtWidgets.QLabel(Form)
        self.lb_frame_idx.setObjectName("lb_frame_idx")
        self.horizontalLayout_6.addWidget(self.lb_frame_idx)
        self.sb_frame_idx = QtWidgets.QSpinBox(Form)
        self.sb_frame_idx.setMinimum(0)
        self.sb_frame_idx.setMaximum(100000000)
        self.sb_frame_idx.setObjectName("sb_frame_idx")
        self.horizontalLayout_6.addWidget(self.sb_frame_idx)
        self.cb_set_frame_idx_na = QtWidgets.QCheckBox(Form)
        self.cb_set_frame_idx_na.setObjectName("cb_set_frame_idx_na")
        self.horizontalLayout_6.addWidget(self.cb_set_frame_idx_na)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.leComment = QtWidgets.QPlainTextEdit(Form)
        self.leComment.setObjectName("leComment")
        self.verticalLayout.addWidget(self.leComment)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.pbCancel = QtWidgets.QPushButton(Form)
        self.pbCancel.setObjectName("pbCancel")
        self.horizontalLayout.addWidget(self.pbCancel)
        self.pbOK = QtWidgets.QPushButton(Form)
        self.pbOK.setDefault(True)
        self.pbOK.setObjectName("pbOK")
        self.horizontalLayout.addWidget(self.pbOK)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Edit event"))
        self.label.setText(_translate("Form", "Time"))
        self.pb_set_to_current_time.setText(_translate("Form", "Set to current time"))
        self.cb_set_time_na.setText(_translate("Form", "Set NA"))
        self.lb_image_idx.setText(_translate("Form", "Image index"))
        self.pb_set_to_current_image_index.setText(_translate("Form", "Set to current image index"))
        self.lbSubject.setText(_translate("Form", "Subject"))
        self.label_2.setText(_translate("Form", "Code"))
        self.lb_frame_idx.setText(_translate("Form", "Frame index"))
        self.cb_set_frame_idx_na.setText(_translate("Form", "Set NA"))
        self.label_4.setText(_translate("Form", "Comment"))
        self.pbCancel.setText(_translate("Form", "Cancel"))
        self.pbOK.setText(_translate("Form", "OK"))
